/* Type information for objc/objc-next-runtime-abi-01.cc.
   Copyright (C) 2004-2022 Free Software Foundation, Inc.

This file is part of GCC.

GCC is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 3, or (at your option) any later
version.

GCC is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with GCC; see the file COPYING3.  If not see
<http://www.gnu.org/licenses/>.  */

/* This file is machine generated.  Do not edit.  */

/* GC roots.  */

EXPORTED_CONST struct ggc_root_tab gt_ggc_r_gt_objc_objc_next_runtime_abi_01_h[] = {
  {
    &objc_eh_personality_decl,
    1,
    sizeof (objc_eh_personality_decl),
    &gt_ggc_mx_tree_node,
    &gt_pch_nx_tree_node
  },
  {
    &V1_Property_decl,
    1,
    sizeof (V1_Property_decl),
    &gt_ggc_mx_tree_node,
    &gt_pch_nx_tree_node
  },
  {
    &V1_ProtocolExt_decl,
    1,
    sizeof (V1_ProtocolExt_decl),
    &gt_ggc_mx_tree_node,
    &gt_pch_nx_tree_node
  },
  {
    &V1_Protocol_OPT_CLS_METHODS_decl,
    1,
    sizeof (V1_Protocol_OPT_CLS_METHODS_decl),
    &gt_ggc_mx_tree_node,
    &gt_pch_nx_tree_node
  },
  {
    &V1_Protocol_OPT_NST_METHODS_decl,
    1,
    sizeof (V1_Protocol_OPT_NST_METHODS_decl),
    &gt_ggc_mx_tree_node,
    &gt_pch_nx_tree_node
  },
  {
    &objc_class_ext_template,
    1,
    sizeof (objc_class_ext_template),
    &gt_ggc_mx_tree_node,
    &gt_pch_nx_tree_node
  },
  {
    &objc_protocol_extension_template,
    1,
    sizeof (objc_protocol_extension_template),
    &gt_ggc_mx_tree_node,
    &gt_pch_nx_tree_node
  },
  {
    &objc_v1_property_template,
    1,
    sizeof (objc_v1_property_template),
    &gt_ggc_mx_tree_node,
    &gt_pch_nx_tree_node
  },
  LAST_GGC_ROOT_TAB
};

EXPORTED_CONST struct ggc_root_tab gt_pch_rs_gt_objc_objc_next_runtime_abi_01_h[] = {
  { &class_reference_idx, 1, sizeof (class_reference_idx), NULL, NULL },
  LAST_GGC_ROOT_TAB
};

